@@ -1,36 +1,38 @@
 #!/usr/bin/python
 
+from pychess.System import glock
+from pychess.System.GtkWorker import GtkWorker
+from pychess.System.Log import log
+from pychess.System.ThreadPool import pool
+from pychess.System.prefix import addDataPrefix, isInstalled
+from pychess.System.repeat import repeat_sleep
+from pychess.Utils.book import getOpenings
+from pychess.Utils.const import *
+from pychess.Utils.lutils import leval, lsearch
+from pychess.Utils.lutils.LBoard import LBoard
+from pychess.Utils.lutils.lmove import determineAlgebraicNotation, parseSAN, \
+    listToSan, toSAN, parseAny, toLAN
+from pychess.Utils.lutils.lsearch import alphaBeta
+from pychess.Utils.lutils.validator import validateMove
+from pychess.Utils.repr import reprResult_long, reprReason_long
+from pychess.ic import FICSConnection
+from pychess.widgets import LogDialog
+from time import time
+from urllib import urlopen, urlencode
+import email.Utils
+import gettext
+import gtk.glade
+import math
+import pychess
+import random
+import signal
+import subprocess
+import sys
+
 print "feature done=0"
 
-from time import time
-import sys, os
-import random, math
-import subprocess
-import email.Utils
-import gettext
-from urllib import urlopen, urlencode
-
-from pychess.System.prefix import addDataPrefix
 gettext.install("pychess", localedir=addDataPrefix("lang"), unicode=1)
 
-import pychess
-from pychess.Utils.const import *
-from pychess.Utils.repr import reprResult_long, reprReason_long
-from pychess.Utils.book import getOpenings
-from pychess.Utils.lutils.lsearch import alphaBeta
-from pychess.Utils.lutils import lsearch
-from pychess.Utils.lutils.lmove import *
-from pychess.Utils.lutils.LBoard import LBoard
-from pychess.Utils.lutils import leval
-from pychess.Utils.lutils.validator import validateMove
-
-from pychess.System.GtkWorker import GtkWorker
-from pychess.System import glock
-from pychess.System.repeat import repeat_sleep
-from pychess.System.ThreadPool import pool
-from pychess.System.Log import log
-
-from pychess.ic.FICSConnection import FICSConnection
 
 class PyChess:
     
@@ -216,7 +218,7 @@ class PyChess:
             lsearch.nodes = 0
             lsearch.movesearches = 0
     
-    def __analyze2 ():
+    def __analyze2 (self):
         import profile
         profile.runctx("self.__analyze2()", locals(), globals(), "/tmp/pychessprofile")
         from pstats import Stats
@@ -457,7 +459,7 @@ class PyChessFICS(PyChess):
             return int(tri)
         elif tri > mode:
             return int(math.ceil(tri))
-        return int(math.round(tri))
+        return int(round(tri))
     
     def sendChallenges(self):
         if self.connection.bm.isPlaying():
@@ -510,7 +512,8 @@ class PyChessFICS(PyChess):
         #color = random.choice(self.colors)
         self.extendlog(["Seeking %d %d" % (minute, gain)])
         self.connection.glm.seek(minute, gain, True)
-        opps = random.sample(self.connection.glm.getPlayerlist(), self.challenges)
+        opps = random.sample(self.connection.players.get_online_playernames(),
+                             self.challenges)
         self.extendlog("Challenging %s" % op for op in opps)
         for player in opps:
             self.connection.om.challenge(player, minute, gain, True)
@@ -529,7 +532,7 @@ class PyChessFICS(PyChess):
         self.connection.glm.connect("removePlayer", self.__onRemovePlayer)
         self.connection.cm.connect("privateMessage", self.__onTell)
         self.connection.alm.connect("logOut", self.__onLogOut)
-        self.connection.bm.connect("playBoardCreated", self.__onPlayBoardCreated)
+        self.connection.bm.connect("playGameCreated", self.__onGameCreated)
         self.connection.bm.connect("curGameEnded", self.__onGameEnded)
         self.connection.bm.connect("boardUpdate", self.__onBoardUpdate)
         self.connection.om.connect("onChallengeAdd", self.__onChallengeAdd)
@@ -679,19 +682,19 @@ class PyChessFICS(PyChess):
     # Playing
     #===========================================================================
     
-    def __onPlayBoardCreated (self, boardManager, board):
+    def __onGameCreated (self, boardManager, ficsgame):
         
-        self.mytime = int(board["mins"])*60
-        self.increment = int(board["incr"])
-        self.gameno = board["gameno"]
+        self.mytime = int(ficsgame.min)*60
+        self.increment = int(ficsgame.inc)
+        self.gameno = ficsgame.gameno
         self.lastPly = -1
         
         self.acceptedTimesettings.append((self.mytime, self.increment))
         
         self.tellHome("Starting a game (%s, %s) gameno: %s" %
-                (board["wname"], board["bname"], board["gameno"]))
+                (ficsgame.wplayer.name, ficsgame.bplayer.name, ficsgame.gameno))
         
-        if board["bname"].lower() == self.connection.getUsername().lower():
+        if ficsgame.bplayer.name.lower() == self.connection.getUsername().lower():
             self.playingAs = BLACK
         else:
             self.playingAs = WHITE
@@ -707,8 +710,8 @@ class PyChessFICS(PyChess):
         self.worker.connect("done", self.__onMoveCalculated)
         self.worker.execute()
     
-    def __onGameEnded (self, boardManager, gameno, wname, bname, result, reason):
-        self.tellHome(reprResult_long[result] + " " + reprReason_long[reason])
+    def __onGameEnded (self, boardManager, ficsgame):
+        self.tellHome(reprResult_long[ficsgame.result] + " " + reprReason_long[ficsgame.reason])
         lsearch.searching = False
         if self.worker:
             self.worker.cancel()
@@ -800,10 +803,9 @@ if __name__ == "__main__":
     elif len(sys.argv) == 5 and sys.argv[1] == "fics":
         pychess = PyChessFICS(*sys.argv[2:])
         
-        import signal, gtk
+        
         signal.signal(signal.SIGINT, gtk.main_quit)
-        import gettext, gtk.glade
-        from pychess.System.prefix import addDataPrefix, getDataPrefix, isInstalled
+        
         if isInstalled():
             gettext.install("pychess", unicode=1)
             gtk.glade.bindtextdomain("pychess")
@@ -813,9 +815,9 @@ if __name__ == "__main__":
         gtk.glade.textdomain("pychess")
         
         # Start logging
-        from pychess.System.Log import log
+        
         log.debug("Started\n")
-        from pychess.widgets import LogDialog
+        
         LogDialog.show()
         
     else:
