@@ -1,9 +1,8 @@
-from pychess.Utils.Cord import Cord
-from pychess.Utils.Piece import Piece
-from pychess.Utils.Move import Move
+from ChessFile import ChessFile, LoadingError
+from pychess.Utils.GameModel import GameModel
 from pychess.Utils.const import *
-from pychess.Utils.lutils.leval import evaluateComplete
 from pychess.Utils.logic import getStatus, repetitionCount
+from pychess.Utils.lutils.leval import evaluateComplete
 
 __label__ = _("Chess Position")
 __endings__ = "epd",
@@ -69,11 +68,10 @@ def save (file, model):
 def load (file):
     return EpdFile ([line for line in map(str.strip, file) if line])
 
-from ChessFile import ChessFile, LoadingError
 
 class EpdFile (ChessFile):
     
-    def loadToModel (self, gameno, position, model=None):
+    def loadToModel (self, gameno, position, model=None, quick_parse=True):
         if not model: model = GameModel()
         
         fieldlist = self.games[gameno].split(" ")
@@ -104,6 +102,7 @@ class EpdFile (ChessFile):
         else: fen += " 1"
         
         model.boards = [model.variant.board(setup=fen)]
+        model.variations = [model.boards]
         model.status = WAITING_TO_START
         
         # rc is kinda broken
