@@ -1,14 +1,15 @@
-from const import KING, QUEEN, ROOK, BISHOP, KNIGHT, PAWN
-from const import reprSign, reprColor, reprPiece
+from pychess.Utils.const import KING, QUEEN, ROOK, BISHOP, KNIGHT, PAWN
+from pychess.Utils.repr import reprSign, reprColor, reprPiece
+
 
 class Piece:
     def __init__ (self, color, piece):
         self.color = color
         self.piece = piece
-    	
-    	self.opacity = 1.0
-    	self.x = None
-    	self.y = None
+        
+        self.opacity = 1.0
+        self.x = None
+        self.y = None
     
     # Sign is a deprecated synonym for piece
     def _set_sign (self, sign):
