@@ -1,7 +1,7 @@
-
 from array import array
 
 from pychess.Utils.const import *
+from pychess.Utils.repr import reprColor
 from ldata import *
 from attack import isAttacked
 from bitboard import *
@@ -75,13 +75,16 @@ class LBoard:
         self.pawnhash = 0
         
         ########################################################################
-        #  The format of history is a list of tupples of the following fields  #
+        #  The format of history is a list of tuples of the following fields   #
         #  move:       The move that was applied to get the position           #
         #  tpiece:     The piece the move captured, == EMPTY for normal moves  #
         #  enpassant:  cord which can be captured by enpassant or None         #
         #  castling:   The castling availability in the position               #
         #  hash:       The hash of the position                                #
         #  fifty:      A counter for the fifty moves rule                      #
+        #                                                                      #
+        #  Early entries may be None instead of tuples if the information is   #
+        #  not available (e.g. if the board was loaded from a position).       #
         ########################################################################
         self.history = []
 
@@ -144,6 +147,9 @@ class LBoard:
                                 "Pos(%d) - %s") % (epChr, fenstr.rfind(epChr), \
                                  fenstr)
         
+        if (not 'k' in pieceChrs) or (not 'K' in pieceChrs):
+            raise SyntaxError, "FEN needs at least 'k' and 'K' in piece placement field."
+
         # Reset this board
         
         self._reset()
