@@ -1,11 +1,11 @@
-
 from threading import RLock
 from gobject import *
+import re
 from time import time
+from pychess.ic import *
 from pychess.Utils.const import *
 from pychess.Utils.Rating import Rating
 from pychess.System.Log import log
-import re
 
 types = "(?:blitz|standard|lightning|wild|bughouse|crazyhouse|suicide|losers|atomic)"
 rated = "(rated|unrated)"
@@ -20,18 +20,6 @@ times = "[, ]*".join("(?:(\d+) %s)?" % s for s in ("days", "hrs", "mins", "secs"
 # "73 days, 5 hrs, 55 mins"
 # ('73', '5', '55', None)
 
-type2Type = {
-    "blitz": TYPE_BLITZ,
-    "standard": TYPE_STANDARD,
-    "lightning": TYPE_LIGHTNING,
-    "wild": TYPE_WILD,
-    "bughouse": TYPE_BUGHOUSE,
-    "crazyhouse": TYPE_CRAZYHOUSE,
-    "suicide": TYPE_SUICIDE,
-    "losers": TYPE_LOSERS,
-    "atomic": TYPE_ATOMIC
-}
-
 class FingerObject:
     
     def __init__ (self, name = ""):
@@ -46,6 +34,7 @@ class FingerObject:
         self.__totalTimeOnline = 0
         self.__created = 0 # Since field from % of life online
         self.__email = ""
+        self.__sanctions = ""
         self.__adminLevel = ""
         self.__timeseal = False
         self.__notes = [""]*10
@@ -54,7 +43,7 @@ class FingerObject:
         self.__opponent = ""
         self.__silence = False
         self.__titles = None
-       
+        
         self.__rating = {}
     
     def getName (self):
@@ -101,6 +90,11 @@ class FingerObject:
         """ Returns the email adress of the user.
             This will probably only be set for the logged in user """
         return self.__email
+    
+    def getSanctions(self):
+        """ Returns any sanctions the user has against them. This is usually
+            an empty string """
+        return self.__sanctions
 
     def getAdminLevel(self):
         """ Returns the admin level as a string
@@ -142,7 +136,7 @@ class FingerObject:
         return self.__rating[type]
     
     def getTitles(self):
-        return self.__titles    
+        return self.__titles
     
     def setName(self, value):
         self.__name = value
@@ -173,13 +167,13 @@ class FingerObject:
     def setCreated(self, value):
         """ Use relative seconds """
         self.__created = value
-
-    def getStatus(self):
-        return self.__status
     
     def setEmail(self, value):
         self.__email = value
 
+    def setSanctions(self, value):
+        self.__sanctions = value
+        
     def setAdminLevel(self, value):
         self.__adminLevel = value
 
@@ -229,6 +223,7 @@ class FingerManager (GObject):
             "rating +RD +win +loss +draw +total +best",
             "(?P<gametype>%s) +(?P<ratings>.+)" % types,
             "Email *: (?P<email>.+)",
+            "Sanctions *: (?P<sanctions>.+)",
             "Total time online: (?P<tto>.+)",
             "% of life online:  [\d\.]+  \(since (?P<created>.+?)\)", 
             "Timeseal [ \\d] : (?P<timeseal>Off|On)",
@@ -300,18 +295,22 @@ class FingerManager (GObject):
                 finger.setStatus(IC_STATUS_BUSY)
                 finger.setBusyMessage(groupdict["busymessage"])
             elif groupdict["gametype"] != None:
-                type = type2Type[groupdict["gametype"].lower()]
+                gametype = GAME_TYPES_BY_FICS_NAME[groupdict["gametype"].lower()]
                 ratings = groupdict["ratings"].split()
                 del ratings[5] # We don't need the totals field
+                ratings[1] = float(ratings[1])
                 if len(ratings) == 5:
-                    rating = Rating(type, *map(float,ratings))
+                    args = map(int, ratings)
+                    rating = Rating(gametype.rating_type, *args)
                 else:
                     bestTime = self.parseShortDate(ratings[6][1:-1])
-                    args = map(float,ratings[:6]) + [bestTime]
-                    rating = Rating(type, *args)
-                finger.setRating(type, rating)
+                    args = map(int,ratings[:6]) + [bestTime]
+                    rating = Rating(gametype.rating_type, *args)
+                finger.setRating(gametype.rating_type, rating)
             elif groupdict["email"] != None:
                 finger.setEmail(groupdict["email"])
+            elif groupdict["sanctions"] != None:
+                finger.setSanctions(groupdict["sanctions"])
             elif groupdict["tto"] != None:
                 finger.setTotalTimeOnline(self.parseTime(groupdict["tto"]))
             elif groupdict["created"] != None:
