@@ -1,22 +1,29 @@
 
 """ This module handles the tabbed layout in PyChess """
 
-import imp, os
-import traceback
-import cStringIO
-
-import gtk, gobject
-
-from pychess.Utils.IconLoader import load_icon
-from pychess.System.Log import log
-from pychess.System import glock, conf, prefix
-from ChessClock import ChessClock
 from BoardControl import BoardControl
-from pydock.PyDockTop import PyDockTop
-from pydock.__init__ import CENTER, EAST, SOUTH
+from ChessClock import ChessClock
+from MenuItemsDict import MenuItemsDict
+from pychess.System import glock, conf, prefix
+from pychess.System.Log import log
+from pychess.System.glock import glock_connect
 from pychess.System.prefix import addUserConfigPrefix
 from pychess.System.uistuff import makeYellow
 from pychess.Utils.GameModel import GameModel
+from pychess.Utils.IconLoader import load_icon
+from pychess.Utils.const import *
+from pychess.Utils.logic import playerHasMatingMaterial, isClaimableDraw
+from pychess.ic.ICGameModel import ICGameModel
+from pydock.PyDockTop import PyDockTop
+from pydock.__init__ import CENTER, EAST, SOUTH
+import cStringIO
+import gtk
+import gobject
+import imp
+import os
+import traceback
+
+
 
 
 ################################################################################
@@ -51,20 +58,12 @@ media_rewind = load_icon(16, "gtk-media-rewind-ltr")
 media_forward = load_icon(16, "gtk-media-forward-ltr")
 media_next = load_icon(16, "gtk-media-next-ltr")
 
-GAME_MENU_ITEMS = ("save_game1", "save_game_as1", "properties1", "close1")
-ACTION_MENU_ITEMS = ("abort", "adjourn", "draw", "pause1", "resume1", "undo1", 
-                     "call_flag", "resign", "ask_to_move")
-VIEW_MENU_ITEMS = ("rotate_board1", "show_sidepanels", "hint_mode", "spy_mode")
-MENU_ITEMS = GAME_MENU_ITEMS + ACTION_MENU_ITEMS + VIEW_MENU_ITEMS
-
 path = prefix.addDataPrefix("sidepanel")
 postfix = "Panel.py"
 files = [f[:-3] for f in os.listdir(path) if f.endswith(postfix)]
 sidePanels = [imp.load_module(f, *imp.find_module(f, [path])) for f in files]
-pref_sidePanels = []
-for panel in sidePanels:
-    if conf.get(panel.__name__, True):
-        pref_sidePanels.append(panel)
+
+dockLocation = addUserConfigPrefix("pydock.xml")
 
 ################################################################################
 # Initialize module variables                                                  #
@@ -117,6 +116,19 @@ class GameWidget (gobject.GObject):
         self.boardvbox = boardvbox
         self.stat_hbox = stat_hbox
         
+        self.menuitems = MenuItemsDict(self)
+        gamemodel.connect("game_started", self.game_started)
+        gamemodel.connect("game_ended", self.game_ended)
+        gamemodel.connect("game_changed", self.game_changed)
+        gamemodel.connect("game_paused", self.game_paused)
+        gamemodel.connect("game_resumed", self.game_resumed)
+        gamemodel.connect("moves_undone", self.moves_undone)
+        gamemodel.connect("game_unended", self.game_unended)
+        gamemodel.connect("players_changed", self.players_changed)
+        self.players_changed(gamemodel)
+        if gamemodel.timemodel:
+            gamemodel.timemodel.connect("zero_reached", self.zero_reached)
+        
         # Some stuff in the sidepanels .load functions might change UI, so we
         # need glock
         # TODO: Really?
@@ -129,6 +141,189 @@ class GameWidget (gobject.GObject):
     def __del__ (self):
         self.board.__del__()
     
+    def _update_menu_abort (self):
+        if self.gamemodel.isEngine2EngineGame():
+            self.menuitems["abort"].sensitive = True
+            self.menuitems["abort"].tooltip = ""
+        elif self.gamemodel.isObservationGame():
+            self.menuitems["abort"].sensitive = False
+        elif isinstance(self.gamemodel, ICGameModel) \
+           and self.gamemodel.status in UNFINISHED_STATES:
+            if self.gamemodel.ply < 2:
+                self.menuitems["abort"].label = _("Abort")
+                self.menuitems["abort"].tooltip = \
+                    _("This game can be automatically aborted without rating loss because there has not yet been two moves made")
+            else:
+                self.menuitems["abort"].label = _("Offer Abort")
+                self.menuitems["abort"].tooltip = \
+                    _("Your opponent must agree to abort the game because there has been two or more moves made")
+            self.menuitems["abort"].sensitive = True
+        else:
+            self.menuitems["abort"].sensitive = False
+            self.menuitems["abort"].tooltip = ""
+
+    def _update_menu_adjourn (self):
+        self.menuitems["adjourn"].sensitive = \
+            isinstance(self.gamemodel, ICGameModel) and \
+            self.gamemodel.status in UNFINISHED_STATES and \
+            not self.gamemodel.isObservationGame() and \
+            not self.gamemodel.hasGuestPlayers()
+        
+        if isinstance(self.gamemodel, ICGameModel) and \
+            self.gamemodel.status in UNFINISHED_STATES and \
+            not self.gamemodel.isObservationGame() and \
+            self.gamemodel.hasGuestPlayers():
+            self.menuitems["adjourn"].tooltip = \
+                _("This game can not be adjourned because one or both players are guests")
+        else:
+            self.menuitems["adjourn"].tooltip = ""
+
+    def _update_menu_draw (self):
+        self.menuitems["draw"].sensitive = self.gamemodel.status in UNFINISHED_STATES \
+            and not self.gamemodel.isObservationGame()
+        
+        def can_win (color):
+            if self.gamemodel.timemodel:
+                return playerHasMatingMaterial(self.gamemodel.boards[-1], color) and \
+                    self.gamemodel.timemodel.getPlayerTime(color) > 0
+            else:
+                return playerHasMatingMaterial(self.gamemodel.boards[-1], color)
+        if isClaimableDraw(self.gamemodel.boards[-1]) or not \
+                (can_win(self.gamemodel.players[0].color) or \
+                 can_win(self.gamemodel.players[1].color)):
+            self.menuitems["draw"].label = _("Claim Draw")
+        
+    def _update_menu_resign (self):
+        self.menuitems["resign"].sensitive = self.gamemodel.status in UNFINISHED_STATES \
+            and not self.gamemodel.isObservationGame()
+    
+    def _update_menu_pause_and_resume (self):
+        self.menuitems["pause1"].sensitive = self.gamemodel.status == RUNNING \
+            and (self.gamemodel.isEngine2EngineGame() or not self.gamemodel.isObservationGame())
+        self.menuitems["resume1"].sensitive = self.gamemodel.status == PAUSED \
+            and (self.gamemodel.isEngine2EngineGame() or not self.gamemodel.isObservationGame())
+        # TODO: if IC game is over and opponent is available, enable Resume
+    
+    def _update_menu_undo (self):
+        if self.gamemodel.isObservationGame():
+            self.menuitems["undo1"].sensitive = False
+        elif isinstance(self.gamemodel, ICGameModel):
+            if self.gamemodel.status in UNFINISHED_STATES and self.gamemodel.ply > 0:
+                self.menuitems["undo1"].sensitive = True
+            else:
+                self.menuitems["undo1"].sensitive = False
+        elif self.gamemodel.ply > 0 \
+           and self.gamemodel.status in UNDOABLE_STATES + (RUNNING,):
+                self.menuitems["undo1"].sensitive = True
+        else:
+            self.menuitems["undo1"].sensitive = False
+    
+    def _update_menu_ask_to_move (self):
+        if self.gamemodel.isObservationGame():
+            self.menuitems["ask_to_move"].sensitive = False
+        elif isinstance(self.gamemodel, ICGameModel):
+            self.menuitems["ask_to_move"].sensitive = False
+        elif self.gamemodel.waitingplayer.__type__ == LOCAL \
+           and self.gamemodel.status in UNFINISHED_STATES \
+           and self.gamemodel.status != PAUSED:
+            self.menuitems["ask_to_move"].sensitive = True
+        else:
+            self.menuitems["ask_to_move"].sensitive = False
+    
+    def game_started (self, gamemodel):
+        self._update_menu_abort()
+        self._update_menu_adjourn()
+        self._update_menu_draw()
+        self._update_menu_pause_and_resume()
+        self._update_menu_resign()
+        self._update_menu_undo()
+        self._update_menu_ask_to_move()
+        
+        activate_hint = False
+        activate_spy = False
+        if HINT in gamemodel.spectators and not \
+                (isinstance(self.gamemodel, ICGameModel) and
+                 self.gamemodel.isObservationGame() is False):
+            activate_hint = True
+        if SPY in gamemodel.spectators and not \
+                (isinstance(self.gamemodel, ICGameModel) and
+                 self.gamemodel.isObservationGame() is False):
+            activate_spy = True
+        self.menuitems["hint_mode"].active = activate_hint
+        self.menuitems["hint_mode"].sensitive = HINT in gamemodel.spectators
+        self.menuitems["spy_mode"].active = activate_spy
+        self.menuitems["spy_mode"].sensitive = SPY in gamemodel.spectators
+    
+    def game_ended (self, gamemodel, reason):
+        for item in self.menuitems:
+            self.menuitems[item].sensitive = False
+        self._update_menu_undo()
+    
+    def game_changed (self, gamemodel):
+        self._update_menu_abort()
+        self._update_menu_ask_to_move()
+        self._update_menu_draw()
+        self._update_menu_pause_and_resume()
+        self._update_menu_undo()
+    
+    def game_paused (self, gamemodel):
+        self._update_menu_pause_and_resume()
+        self._update_menu_undo()
+        self._update_menu_ask_to_move()
+    
+    def game_resumed (self, gamemodel):
+        self._update_menu_pause_and_resume()
+        self._update_menu_undo()
+        self._update_menu_ask_to_move()
+    
+    def moves_undone (self, gamemodel, moves):
+        self.game_changed(gamemodel)
+    
+    def game_unended (self, gamemodel):
+        self.game_started(gamemodel)
+    
+    def players_changed (self, gamemodel):
+        for player in gamemodel.players:
+            self.name_changed(player)
+            # Notice that this may connect the same player many times. In
+            # normal use that shouldn't be a problem.
+            glock_connect(player, "name_changed", self.name_changed)
+    
+    @property
+    def display_text (self):
+        vs = " " + _("vs") + " "
+        if isinstance(self.gamemodel, ICGameModel):
+            ficsgame = self.gamemodel.ficsgame
+            t = vs.join((ficsgame.wplayer.long_name(game_type=ficsgame.game_type),
+                         ficsgame.bplayer.long_name(game_type=ficsgame.game_type)))
+        else:
+            t = vs.join(map(repr, self.gamemodel.players))
+        
+        if self.gamemodel.display_text != "":
+            t += " " + self.gamemodel.display_text
+        return t
+    
+    def name_changed (self, player):
+        newText = self.display_text
+        if newText != self.getTabText():
+            self.setTabText(newText)
+    
+    def zero_reached (self, timemodel, color):
+        if self.gamemodel.status not in UNFINISHED_STATES: return
+        
+        if self.gamemodel.players[0].__type__ == LOCAL \
+           and self.gamemodel.players[1].__type__ == LOCAL:
+            self.menuitems["call_flag"].sensitive = True
+            return
+        
+        for player in self.gamemodel.players:
+            opplayercolor = BLACK if player == self.gamemodel.players[WHITE] else WHITE
+            if player.__type__ == LOCAL and opplayercolor == color:
+                log.debug("gamewidget.zero_reached: LOCAL player=%s, color=%s\n" % \
+                          (repr(player), str(color)))
+                self.menuitems["call_flag"].sensitive = True
+                break
+    
     def initTabcontents(self):
         tabcontent = createAlignment(gtk.Notebook().props.tab_vborder,0,0,0)
         hbox = gtk.HBox()
@@ -219,14 +414,19 @@ class GameWidget (gobject.GObject):
         return statusbar, stat_hbox
     
     def setLocked (self, locked):
-        """ Makes the board insensitive and turns of the tab ready indicator """
+        """ Makes the board insensitive and turns off the tab ready indicator """
+        log.debug("GameWidget.setLocked: %s locked=%s\n" % (self.gamemodel.players, str(locked)))
         self.board.setLocked(locked)
         if not self.tabcontent.get_children(): return
+        if len(self.tabcontent.child.get_children()) < 2:
+            log.warn("GameWidget.setLocked: Not removing last tabcontent child\n")
+            return
         self.tabcontent.child.remove(self.tabcontent.child.get_children()[0])
         if not locked:
             self.tabcontent.child.pack_start(createImage(light_on), expand=False)
         else: self.tabcontent.child.pack_start(createImage(light_off), expand=False)
         self.tabcontent.show_all()
+        log.debug("GameWidget.setLocked: %s: returning\n" % self.gamemodel.players)
     
     def setTabText (self, text):
         self.tabcontent.child.get_children()[1].set_text(text)
@@ -291,6 +491,9 @@ class GameWidget (gobject.GObject):
     
     def hideMessage (self):
         self.messageSock.hide()
+        if self == cur_gmwidg():
+            notebooks["messageArea"].hide()
+
 
 ################################################################################
 # Main handling of gamewidgets                                                 #
@@ -373,7 +576,6 @@ def _ensureReadForGameWidgets ():
     dockAlign.show()
     dock.show()
     
-    dockLocation = addUserConfigPrefix("pydock.xml")
     for panel in sidePanels:
         hbox = gtk.HBox()
         pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(panel.__icon__, 16, 16)
@@ -435,18 +637,14 @@ def _ensureReadForGameWidgets ():
         leaf.setDockable(False)
         
         # NE
-        leaf = leaf.dock(docks["historyPanel"][1], EAST, docks["historyPanel"][0], "historyPanel")
-        conf.set("historyPanel", True)
+        leaf = leaf.dock(docks["annotationPanel"][1], EAST, docks["annotationPanel"][0], "annotationPanel")
+        leaf = leaf.dock(docks["historyPanel"][1], CENTER, docks["historyPanel"][0], "historyPanel")
         leaf = leaf.dock(docks["scorePanel"][1], CENTER, docks["scorePanel"][0], "scorePanel")
-        conf.set("scorePanel", True)
         
         # SE
         leaf = leaf.dock(docks["bookPanel"][1], SOUTH, docks["bookPanel"][0], "bookPanel")
-        conf.set("bookPanel", True)
         leaf = leaf.dock(docks["commentPanel"][1], CENTER, docks["commentPanel"][0], "commentPanel")
-        conf.set("commentPanel", True)
         leaf = leaf.dock(docks["chatPanel"][1], CENTER, docks["chatPanel"][0], "chatPanel")
-        conf.set("chatPanel", True)
     
     def unrealize (dock):
         # unhide the panel before saving so its configuration is saved correctly
@@ -495,7 +693,7 @@ def attachGameWidget (gmwidg):
     def callback (notebook, gpointer, page_num, gmwidg):
         if notebook.get_nth_page(page_num) == gmwidg.notebookKey:
             gmwidg.emit("infront")
-    headbook.connect("switch-page", callback, gmwidg)
+    headbook.connect_after("switch-page", callback, gmwidg)
     gmwidg.emit("infront")
     
     messageSockAlign = createAlignment(4,4,0,4)
