@@ -1,7 +1,6 @@
 import os.path
 import gettext
 import locale
-from time import sleep
 from cStringIO import StringIO
 from operator import attrgetter
 from itertools import groupby
@@ -22,6 +21,8 @@ from pychess.Utils.IconLoader import load_icon
 from pychess.Utils.GameModel import GameModel
 from pychess.Utils.TimeModel import TimeModel
 from pychess.Utils.const import *
+from pychess.Utils.repr import localReprSign
+from pychess.Utils.lutils.LBoard import LBoard
 from pychess.System import uistuff
 from pychess.System.Log import log
 from pychess.System import conf
@@ -31,7 +32,7 @@ from pychess.Players.Human import Human
 from pychess.widgets import BoardPreview
 from pychess.widgets import ionest
 from pychess.widgets import ImageMenu
-from pychess.Savers import pgn
+from pychess.Savers import fen, pgn
 from pychess.Variants import variants
 from pychess.Variants.normal import NormalChess
 
@@ -162,13 +163,22 @@ class _GameInitializationMode:
                 variant = conf.get("ngvariant1", FISCHERRANDOMCHESS)
             else:
                 variant = conf.get("ngvariant2", LOSERSCHESS)
+            variant1 = conf.get("ngvariant1", FISCHERRANDOMCHESS)
+            cls.widgets["playVariant1Radio"].set_tooltip_text(variants[variant1].__desc__)            
+            variant2 = conf.get("ngvariant2", LOSERSCHESS)
+            cls.widgets["playVariant2Radio"].set_tooltip_text(variants[variant2].__desc__)
             uistuff.updateCombo(cls.widgets["blackPlayerCombobox"], playerItems[variant])
             uistuff.updateCombo(cls.widgets["whitePlayerCombobox"], playerItems[variant])
         conf.notify_add("ngvariant1", updateCombos)
         conf.notify_add("ngvariant2", updateCombos)
         cls.widgets["playNormalRadio"].connect("toggled", updateCombos)
+        cls.widgets["playNormalRadio"].set_tooltip_text(variants[NORMALCHESS].__desc__)
         cls.widgets["playVariant1Radio"].connect("toggled", updateCombos)
+        variant1 = conf.get("ngvariant1", FISCHERRANDOMCHESS)
+        cls.widgets["playVariant1Radio"].set_tooltip_text(variants[variant1].__desc__)
         cls.widgets["playVariant2Radio"].connect("toggled", updateCombos)
+        variant2 = conf.get("ngvariant2", LOSERSCHESS)
+        cls.widgets["playVariant2Radio"].set_tooltip_text(variants[variant2].__desc__)
 
         # The "variant" has to come before players, because the engine positions
         # in the user comboboxes can be different in different variants
@@ -259,7 +269,7 @@ class _GameInitializationMode:
         def callback (selection):
             model, iter = selection.get_selected()
             if iter:
-                radiobutton.set_label("%s" % model.get(iter, 0))
+                radiobutton.set_label("%s" % model.get(iter, 0) + _(" chess"))
                 path = model.get_path(iter)
                 variant = pathToVariant[path]
                 conf.set(confid, variant)
@@ -315,8 +325,11 @@ class _GameInitializationMode:
                     playertups.append((ARTIFICIAL, discoverer.initPlayerEngine,
                             (engine, color, diffi, variant, secs, incr), name))
                 else:
-                    playertups.append((LOCAL, Human, (color, ""), _("Human")))
-
+                    if not playertups or playertups[0][0] != LOCAL:
+                        name = conf.get("firstName", _("You"))
+                    else: name = conf.get("secondName", _("Guest"))
+                    playertups.append((LOCAL, Human, (color, name), name))
+            
             if secs > 0:
                 timemodel = TimeModel (secs, incr)
             else: timemodel = None
@@ -488,7 +501,14 @@ class EnterNotationExtension (_GameInitializationMode):
                     text = text.replace(sign, reprSign[i+1])
                 text = str(text)
 
-            ionest.generalStart(gamemodel, p0, p1, (StringIO(text), pgn, 0, -1))
+            # First we try if it's just a FEN string
+            try:
+                LBoard(NORMALCHESS).applyFen(text)
+                loadType = fen
+            except:
+                loadType = pgn
+
+            ionest.generalStart(gamemodel, p0, p1, (StringIO(text), loadType, 0, -1))
         cls._generalRun(_callback)
 
 class ImageButton(gtk.DrawingArea):
@@ -520,6 +540,9 @@ class ImageButton(gtk.DrawingArea):
             self.window.process_updates(True)
 
 def createRematch (gamemodel):
+    """ If gamemodel contains only LOCAL or ARTIFICIAL players, this starts a
+        new game, based on the info in gamemodel """
+    
     if gamemodel.timemodel:
         secs = gamemodel.timemodel.intervals[0][WHITE]
         gain = gamemodel.timemodel.gain
@@ -533,9 +556,9 @@ def createRematch (gamemodel):
     bp = gamemodel.players[BLACK]
 
     if wp.__type__ == LOCAL:
-        player1tup = (wp.__type__, wp.__class__, (BLACK, ""), repr(wp))
+        player1tup = (wp.__type__, wp.__class__, (BLACK, repr(wp)), repr(wp))
         if bp.__type__ == LOCAL:
-            player0tup = (bp.__type__, bp.__class__, (WHITE, ""), repr(bp))
+            player0tup = (bp.__type__, bp.__class__, (WHITE, repr(wp)), repr(bp))
         else:
             binname = bp.engine.path.split("/")[-1]
             if binname == "python":
